# wallet_mcp.py

# Get balance API require to pass the public key.
# Need to save the public key in supabase

import os
import requests
import keyring
import base58
from typing import Optional
from solana.rpc.api import Client
from solders.keypair import Keypair
from mcp import types as mcp_types
from mcp.server.lowlevel import Server
from google.adk.tools.function_tool import FunctionTool
from google.adk.tools.mcp_tool.conversion_utils import adk_to_mcp_tool_type

# === Configuration ===
SOLANA_RPC_URL = "https://api.devnet.solana.com"
LATINUM_API_URL = "https://latinum.ai/api/solana_wallet"
SERVICE_NAME = "latinum-wallet-mcp"
KEY_NAME = "latinum-key"
AIR_DROP_THRESHOLD = 1000
AIR_DROP_AMOUNT = 10000

# === Setup Solana Client ===
client = Client(SOLANA_RPC_URL)

# === Load or Generate Wallet Keypair ===
PRIVATE_KEY_BASE58 = keyring.get_password(SERVICE_NAME, KEY_NAME)
if PRIVATE_KEY_BASE58:
    print("Loaded existing private key from keyring.")
    secret_bytes = base58.b58decode(PRIVATE_KEY_BASE58)
    keypair = Keypair.from_bytes(secret_bytes)
else:
    print("No key found. Generating new wallet...")
    seed = os.urandom(32)
    keypair = Keypair.from_seed(seed)
    PRIVATE_KEY_BASE58 = base58.b58encode(bytes(keypair)).decode("utf-8")
    keyring.set_password(SERVICE_NAME, KEY_NAME, PRIVATE_KEY_BASE58)

public_key = keypair.pubkey()

# === Ensure Wallet Has Funds ===
balance = client.get_balance(public_key).value
if balance < AIR_DROP_THRESHOLD:
    print("Requesting airdrop of 10000")
    try:
        tx_sig = client.request_airdrop(public_key, AIR_DROP_AMOUNT)["result"]
        print(f"Airdrop requested. Tx: {tx_sig}")
    except Exception as e:
        print(f"Airdrop failed: {e}")

# === Display Wallet Info ===
def lamports_to_sol(lamports: int) -> float:
    return lamports / 1_000_000_000

def print_wallet_info():
    print("\nWallet Info")
    print(f"   Public Key: {public_key}")
    balance_lamports = client.get_balance(public_key).value
    print(f"   Balance: {balance_lamports} lamports ({lamports_to_sol(balance_lamports):.9f} SOL)")

    try:
        print("   Recent Transactions:")
        sigs = client.get_signatures_for_address(public_key, limit=5).value
        if not sigs:
            print("     No recent transactions.")
        for s in sigs:
            print(f"     {s.signature} | Slot: {s.slot} | Status: {s.confirmation_status}")
    except Exception as e:
        print(f"   Failed to fetch transactions: {e}")

print_wallet_info()

# === MCP Server Definition ===
def build_mcp_wallet_server() -> Server:
    # Tool: Get signed transaction to pay someone
    def get_signed_transaction(targetWallet: str, amountLamports: int) -> dict:
        try:
            res = requests.post(LATINUM_API_URL, json={
                "sourceWalletPrivate": PRIVATE_KEY_BASE58,
                "targetWallet": targetWallet,
                "amountLamports": amountLamports
            })
            res.raise_for_status()
            data = res.json()

            if not data.get("success") or not data.get("signedTransactionB64"):
                return {
                    "success": False,
                    "message": "Failed to retrieve signed transaction."
                }

            return {
                "success": True,
                "signedTransactionB64": data["signedTransactionB64"],
                "message": f"Signed transaction ready:\n{data['signedTransactionB64']}"
            }
        except Exception as e:
            return {
                "success": False,
                "message": f"Error: {str(e)}"
            }

    # Tool: Get current wallet public key and balance
    def get_wallet_info(_: Optional[str] = None) -> dict:
        try:
            balance = client.get_balance(public_key).value
            return {
                "success": True,
                "address": str(public_key),
                "balanceLamports": balance,
                "message": f"Address: {public_key}\nBalance: {balance} lamports"
            }
        except Exception as e:
            return {
                "success": False,
                "message": f"Error fetching balance: {str(e)}"
            }

    # Create MCP server with tools
    wallet_tool = FunctionTool(get_signed_transaction)
    address_tool = FunctionTool(get_wallet_info)
    server = Server(SERVICE_NAME)

    @server.list_tools()
    async def list_tools():
        return [
            adk_to_mcp_tool_type(wallet_tool),
            adk_to_mcp_tool_type(address_tool),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: dict):
        if name == wallet_tool.name:
            result = await wallet_tool.run_async(args=arguments, tool_context=None)
        elif name == address_tool.name:
            result = await address_tool.run_async(args=arguments, tool_context=None)
        else:
            return [mcp_types.TextContent(type="text", text="Unknown tool")]

        return [mcp_types.TextContent(type="text", text=result.get("message", "Failed."))]

    return server